@@ -1,3 +1,5 @@
+from collections import namedtuple
+
 import numpy as np
 
 from . import lib, utils
@@ -14,18 +16,33 @@ class _AllIndices:
 
 _ALL_INDICES = _AllIndices()
 
+# We may want this as a class eventually (different repr, etc.)
+AxisIndex = namedtuple("AxisIndex", ["size", "index", "cscalar"])
+
 
 class IndexerResolver:
     __slots__ = "obj", "indices", "__weakref__"
 
     def __init__(self, obj, indices):
         self.obj = obj
-        self.indices = self.parse_indices(indices, obj.shape)
+        if indices is Ellipsis:
+            from .scalar import _CScalar
+            from .vector import Vector
+
+            if type(obj) is Vector:
+                self.indices = [AxisIndex(obj._size, _ALL_INDICES, _CScalar(obj._size))]
+            else:
+                self.indices = [
+                    AxisIndex(obj._nrows, _ALL_INDICES, _CScalar(obj._nrows)),
+                    AxisIndex(obj._ncols, _ALL_INDICES, _CScalar(obj._ncols)),
+                ]
+        else:
+            self.indices = self.parse_indices(indices, obj.shape)
 
     @property
     def is_single_element(self):
-        for idx, size in self.indices:
-            if size is not None:
+        for idx in self.indices:
+            if idx.size is not None:
                 return False
         return True
 
@@ -62,27 +79,27 @@ class IndexerResolver:
         if np.issubdtype(typ, np.integer):
             if index >= size:
                 raise IndexError(f"Index out of range: index={index}, size={size}")
-            return _CScalar(int(index)), None
+            return AxisIndex(None, _CScalar(int(index)), None)
         if typ is list:
             pass
         elif typ is slice:
-            if index == slice(None) or index == slice(0, None):
-                # [:] means all indices; use special GrB_ALL indicator
-                return _ALL_INDICES, _CScalar(size)
-            index = list(range(size)[index])
+            from ._slice import slice_to_index
+
+            return slice_to_index(index, size)
+
         elif typ is np.ndarray:
             if len(index.shape) != 1:
                 raise TypeError(f"Invalid number of dimensions for index: {len(index.shape)}")
             if not np.issubdtype(index.dtype, np.integer):
                 raise TypeError(f"Invalid dtype for index: {index.dtype}")
-            return _CArray(index), _CScalar(len(index))
+            return AxisIndex(len(index), _CArray(index), _CScalar(len(index)))
         else:
             from .scalar import Scalar
 
             if typ is Scalar:
                 if index.dtype.name.startswith("F"):
                     raise TypeError(f"An integer is required for indexing.  Got: {index.dtype}")
-                return _CScalar(index), None
+                return AxisIndex(None, _CScalar(index), None)
 
             from .matrix import Matrix, TransposedMatrix
             from .vector import Vector
@@ -111,7 +128,7 @@ class IndexerResolver:
                         f"`x(mask={index.name}) << value`."
                     )
                 raise TypeError(f"Invalid type for index: {typ}; unable to convert to list")
-        return _CArray(index), _CScalar(len(index))
+        return AxisIndex(len(index), _CArray(index), _CScalar(len(index)))
 
     def get_index(self, dim):
         """Return a new IndexerResolver with index for the selected dimension"""
@@ -179,7 +196,7 @@ class AmbiguousAssignOrExtract:
         scalar = self.parent._extract_element(self.resolved_indexes, name="s_extract")
         return scalar.value
 
-    def new(self, *, dtype=None, mask=None, input_mask=None, name=None):
+    def new(self, dtype=None, *, mask=None, input_mask=None, name=None):
         """
         Force extraction of the indexes into a new object
         dtype and mask are the only controllable parameters.
@@ -187,7 +204,7 @@ class AmbiguousAssignOrExtract:
         if self.resolved_indexes.is_single_element:
             if mask is not None or input_mask is not None:
                 raise TypeError("mask is not allowed for single element extraction")
-            return self.parent._extract_element(self.resolved_indexes, dtype=dtype, name=name)
+            return self.parent._extract_element(self.resolved_indexes, dtype, name=name)
         else:
             if input_mask is not None:
                 if mask is not None:
@@ -197,7 +214,7 @@ class AmbiguousAssignOrExtract:
                 _check_mask(input_mask, output=self.parent)
                 mask = self._input_mask_to_mask(input_mask)
             delayed_extractor = self.parent._prep_for_extract(self.resolved_indexes)
-            return delayed_extractor.new(dtype=dtype, mask=mask, name=name)
+            return delayed_extractor.new(dtype, mask=mask, name=name)
 
     def __eq__(self, other):
         if not self.resolved_indexes.is_single_element:
@@ -232,8 +249,8 @@ class AmbiguousAssignOrExtract:
         from .vector import Vector
 
         if type(input_mask.mask) is Vector and type(self.parent) is not Vector:
-            (_, rowsize), (_, colsize) = self.resolved_indexes.indices
-            if rowsize is None:
+            rowidx, colidx = self.resolved_indexes.indices
+            if rowidx.size is None:
                 if self.parent._ncols != input_mask.mask._size:
                     raise ValueError(
                         "Size of `input_mask` Vector does not match ncols of Matrix:\n"
@@ -241,7 +258,7 @@ class AmbiguousAssignOrExtract:
                         f"{self.parent._ncols} != {input_mask.mask._size}"
                     )
                 mask_expr = input_mask.mask._prep_for_extract(self.resolved_indexes.get_index(1))
-            elif colsize is None:
+            elif colidx.size is None:
                 if self.parent._nrows != input_mask.mask._size:
                     raise ValueError(
                         "Size of `input_mask` Vector does not match nrows of Matrix:\n"
@@ -342,7 +359,7 @@ class InfixExprBase:
         self.right = right
         self._value = None
 
-    def new(self, *, dtype=None, mask=None, name=None):
+    def new(self, dtype=None, *, mask=None, name=None):
         if (
             mask is None
             and self._value is not None
@@ -353,15 +370,20 @@ class InfixExprBase:
                 rv.name = name
             self._value = None
             return rv
-        # Rely on the default operator for the method
-        expr = getattr(self.left, self.method_name)(self.right)
-        return expr.new(dtype=dtype, mask=mask, name=name)
+        expr = self._to_expr()
+        return expr.new(dtype, mask=mask, name=name)
 
     dup = new
 
     def _to_expr(self):
-        # Rely on the default operator for the method
-        return getattr(self.left, self.method_name)(self.right)
+        if self._value is None:
+            # Rely on the default operator for the method
+            self._value = getattr(self.left, self.method_name)(self.right)
+        return self._value
+
+    def _get_value(self, attr=None, default=None):
+        expr = self._to_expr()
+        return expr._get_value(attr=attr, default=default)
 
     def _format_expr(self):
         return f"{self.left.name} {self._infix} {self.right.name}"
