@@ -1,4 +1,5 @@
 import itertools
+import warnings
 
 import numpy as np
 
@@ -20,7 +21,7 @@ from .utils import (
     values_to_numpy_buffer,
     wrapdoc,
 )
-from .vector import Vector, VectorExpression
+from .vector import Vector, VectorExpression, _VectorAsMatrix
 
 ffi_new = ffi.new
 
@@ -156,7 +157,7 @@ class Matrix(BaseType):
             return False
 
         matches = self.ewise_mult(other, binary.isclose(rel_tol, abs_tol)).new(
-            dtype=bool, name="M_isclose"
+            bool, name="M_isclose"
         )
         # ewise_mult performs intersection, so nvals will indicate mismatched empty values
         if matches._nvals != self._nvals:
@@ -211,7 +212,7 @@ class Matrix(BaseType):
         self._nrows = nrows.scalar.value
         self._ncols = ncols.scalar.value
 
-    def to_values(self, *, dtype=None):
+    def to_values(self, dtype=None):
         """
         GrB_Matrix_extractTuples
         Extract the rows, columns and values as a 3-tuple of numpy arrays
@@ -263,7 +264,7 @@ class Matrix(BaseType):
         dup_op_given = dup_op is not None
         if not dup_op_given:
             dup_op = binary.plus
-        dup_op = get_typed_op(dup_op, self.dtype)
+        dup_op = get_typed_op(dup_op, self.dtype, kind="binary")
         if dup_op.opclass == "Monoid":
             dup_op = dup_op.binaryop
         else:
@@ -271,7 +272,7 @@ class Matrix(BaseType):
 
         rows = _CArray(rows)
         columns = _CArray(columns)
-        values = _CArray(values, dtype=self.dtype)
+        values = _CArray(values, self.dtype)
         call(
             f"GrB_Matrix_build_{self.dtype.name}",
             [self, rows, columns, values, _CScalar(n), dup_op],
@@ -280,7 +281,7 @@ class Matrix(BaseType):
         if not dup_op_given and self._nvals < n:
             raise ValueError("Duplicate indices found, must provide `dup_op` BinaryOp")
 
-    def dup(self, *, dtype=None, mask=None, name=None):
+    def dup(self, dtype=None, *, mask=None, name=None):
         """
         GrB_Matrix_dup
         Create a new Matrix by duplicating this one
@@ -289,7 +290,7 @@ class Matrix(BaseType):
             if dtype is None:
                 dtype = self.dtype
             rv = Matrix.new(dtype, nrows=self._nrows, ncols=self._ncols, name=name)
-            rv(mask=mask)[:, :] << self
+            rv(mask=mask)[...] = self
         else:
             new_mat = ffi_new("GrB_Matrix*")
             rv = Matrix(new_mat, self.dtype, name=name)
@@ -317,10 +318,8 @@ class Matrix(BaseType):
         new_matrix = ffi_new("GrB_Matrix*")
         dtype = lookup_dtype(dtype)
         rv = cls(new_matrix, dtype, name=name)
-        if type(nrows) is not _CScalar:
-            nrows = _CScalar(nrows)
-        if type(ncols) is not _CScalar:
-            ncols = _CScalar(ncols)
+        nrows = _CScalar(nrows)
+        ncols = _CScalar(ncols)
         call("GrB_Matrix_new", [_Pointer(rv), dtype, nrows, ncols])
         rv._nrows = nrows.scalar.value
         rv._ncols = ncols.scalar.value
@@ -332,11 +331,11 @@ class Matrix(BaseType):
         rows,
         columns,
         values,
+        dtype=None,
         *,
         nrows=None,
         ncols=None,
         dup_op=None,
-        dtype=None,
         name=None,
     ):
         """Create a new Matrix from the given lists of row indices, column
@@ -409,7 +408,7 @@ class Matrix(BaseType):
         other = self._expect_type(
             other, (Matrix, TransposedMatrix), within=method_name, argname="other"
         )
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         # Per the spec, op may be a semiring, but this is weird, so don't.
         if require_monoid:
             if op.opclass != "BinaryOp" or op.monoid is None:
@@ -418,7 +417,7 @@ class Matrix(BaseType):
                     "Monoid",
                     within=method_name,
                     argname="op",
-                    extra_message="A BinaryOp may be given if require_monoid keyword is False",
+                    extra_message="A BinaryOp may be given if require_monoid keyword is False.",
                 )
         else:
             self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
@@ -445,7 +444,7 @@ class Matrix(BaseType):
         other = self._expect_type(
             other, (Matrix, TransposedMatrix), within=method_name, argname="other"
         )
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         # Per the spec, op may be a semiring, but this is weird, so don't.
         self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
         expr = MatrixExpression(
@@ -468,7 +467,7 @@ class Matrix(BaseType):
         """
         method_name = "mxv"
         other = self._expect_type(other, Vector, within=method_name, argname="other")
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="semiring")
         self._expect_op(op, "Semiring", within=method_name, argname="op")
         expr = VectorExpression(
             method_name,
@@ -492,7 +491,7 @@ class Matrix(BaseType):
         other = self._expect_type(
             other, (Matrix, TransposedMatrix), within=method_name, argname="other"
         )
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="semiring")
         self._expect_op(op, "Semiring", within=method_name, argname="op")
         expr = MatrixExpression(
             method_name,
@@ -518,7 +517,7 @@ class Matrix(BaseType):
         other = self._expect_type(
             other, (Matrix, TransposedMatrix), within=method_name, argname="other"
         )
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         # Per the spec, op may be a semiring, but this is weird, so don't.
         self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
         return MatrixExpression(
@@ -532,7 +531,7 @@ class Matrix(BaseType):
             bt=other._is_transposed,
         )
 
-    def apply(self, op, *, left=None, right=None):
+    def apply(self, op, right=None, *, left=None):
         """
         GrB_Matrix_apply
         Apply UnaryOp to each element of the calling Matrix
@@ -544,7 +543,7 @@ class Matrix(BaseType):
             "apply only accepts UnaryOp with no scalars or BinaryOp with `left` or `right` scalar."
         )
         if left is None and right is None:
-            op = get_typed_op(op, self.dtype)
+            op = get_typed_op(op, self.dtype, kind="unary")
             self._expect_op(
                 op,
                 "UnaryOp",
@@ -567,7 +566,7 @@ class Matrix(BaseType):
                         keyword_name="left",
                         extra_message="Literal scalars also accepted.",
                     )
-            op = get_typed_op(op, self.dtype, left.dtype)
+            op = get_typed_op(op, self.dtype, left.dtype, kind="binary")
             if op.opclass == "Monoid":
                 op = op.binaryop
             else:
@@ -593,7 +592,7 @@ class Matrix(BaseType):
                         keyword_name="right",
                         extra_message="Literal scalars also accepted.",
                     )
-            op = get_typed_op(op, self.dtype, right.dtype)
+            op = get_typed_op(op, self.dtype, right.dtype, kind="binary")
             if op.opclass == "Monoid":
                 op = op.binaryop
             else:
@@ -621,14 +620,14 @@ class Matrix(BaseType):
             bt=self._is_transposed,
         )
 
-    def reduce_rows(self, op=monoid.plus):
+    def reduce_rowwise(self, op=monoid.plus):
         """
         GrB_Matrix_reduce
         Reduce all values in each row, converting the matrix to a vector
         Default op is monoid.lor for boolean and monoid.plus otherwise
         """
-        method_name = "reduce_rows"
-        op = get_typed_op(op, self.dtype)
+        method_name = "reduce_rowwise"
+        op = get_typed_op(op, self.dtype, kind="binary")
         self._expect_op(op, ("BinaryOp", "Monoid", "Aggregator"), within=method_name, argname="op")
         # Using a monoid may be more efficient, so change to one if possible.
         # Also, SuiteSparse doesn't like user-defined binarops here.
@@ -643,14 +642,28 @@ class Matrix(BaseType):
             at=self._is_transposed,
         )
 
-    def reduce_columns(self, op=monoid.plus):
+    def reduce_rows(self, op=monoid.plus):
+        """
+        GrB_Matrix_reduce
+        Reduce all values in each row, converting the matrix to a vector
+        Default op is monoid.lor for boolean and monoid.plus otherwise
+
+        **This function is deprecated.  Please use ``Matrix.reduce_rowwise`` instead.
+        """
+        warnings.warn(
+            "`Matrix.reduce_rows` is deprecated; please use `Matrix.reduce_rowwise` instead",
+            DeprecationWarning,
+        )
+        return self.reduce_rowwise(op)
+
+    def reduce_columnwise(self, op=monoid.plus):
         """
         GrB_Matrix_reduce
         Reduce all values in each column, converting the matrix to a vector
         Default op is monoid.lor for boolean and monoid.plus otherwise
         """
-        method_name = "reduce_columns"
-        op = get_typed_op(op, self.dtype)
+        method_name = "reduce_columnwise"
+        op = get_typed_op(op, self.dtype, kind="binary")
         self._expect_op(op, ("BinaryOp", "Monoid", "Aggregator"), within=method_name, argname="op")
         # Using a monoid may be more efficient, so change to one if possible.
         # Also, SuiteSparse doesn't like user-defined binarops here.
@@ -665,6 +678,20 @@ class Matrix(BaseType):
             at=not self._is_transposed,
         )
 
+    def reduce_columns(self, op=monoid.plus):
+        """
+        GrB_Matrix_reduce
+        Reduce all values in each column, converting the matrix to a vector
+        Default op is monoid.lor for boolean and monoid.plus otherwise
+
+        **This function is deprecated.  Please use ``Matrix.reduce_columnwise`` instead.
+        """
+        warnings.warn(
+            "`Matrix.reduce_columns` is deprecated; please use `Matrix.reduce_columnwise` instead",
+            DeprecationWarning,
+        )
+        return self.reduce_columnwise(op)
+
     def reduce_scalar(self, op=monoid.plus):
         """
         GrB_Matrix_reduce
@@ -672,7 +699,7 @@ class Matrix(BaseType):
         Default op is monoid.lor for boolean and monoid.plus otherwise
         """
         method_name = "reduce_scalar"
-        op = get_typed_op(op, self.dtype)
+        op = get_typed_op(op, self.dtype, kind="binary")
         if op.opclass == "BinaryOp" and op.monoid is not None:
             op = op.monoid
         else:
@@ -699,13 +726,15 @@ class Matrix(BaseType):
             dtype = self.dtype
         else:
             dtype = lookup_dtype(dtype)
-        row, _ = resolved_indexes.indices[0]
-        col, _ = resolved_indexes.indices[1]
+        rowidx, colidx = resolved_indexes.indices
         if self._is_transposed:
-            row, col = col, row
+            rowidx, colidx = colidx, rowidx
         result = Scalar.new(dtype, name=name)
         if (
-            call(f"GrB_Matrix_extractElement_{dtype}", [_Pointer(result), self, row, col])
+            call(
+                f"GrB_Matrix_extractElement_{dtype}",
+                [_Pointer(result), self, rowidx.index, colidx.index],
+            )
             is not NoValue
         ):
             result._is_empty = False
@@ -713,15 +742,13 @@ class Matrix(BaseType):
 
     def _prep_for_extract(self, resolved_indexes):
         method_name = "__getitem__"
-        rows, rowsize = resolved_indexes.indices[0]
-        cols, colsize = resolved_indexes.indices[1]
+        (rowsize, rows, rowscalar), (colsize, cols, colscalar) = resolved_indexes.indices
         if rowsize is None:
             # Row-only selection; GraphBLAS doesn't have this method, so we hack it using transpose
-            row_index = rows
             return VectorExpression(
                 method_name,
                 "GrB_Col_extract",
-                [self, cols, colsize, row_index],
+                [self, cols, colscalar, rows],
                 expr_repr="{0.name}[{3}, [{2} cols]]",
                 size=colsize,
                 dtype=self.dtype,
@@ -729,11 +756,10 @@ class Matrix(BaseType):
             )
         elif colsize is None:
             # Column-only selection
-            col_index = cols
             return VectorExpression(
                 method_name,
                 "GrB_Col_extract",
-                [self, rows, rowsize, col_index],
+                [self, rows, rowscalar, cols],
                 expr_repr="{0.name}[[{2} rows], {3}]",
                 size=rowsize,
                 dtype=self.dtype,
@@ -743,7 +769,7 @@ class Matrix(BaseType):
             return MatrixExpression(
                 method_name,
                 "GrB_Matrix_extract",
-                [self, rows, rowsize, cols, colsize],
+                [self, rows, rowscalar, cols, colscalar],
                 expr_repr="{0.name}[[{2} rows], [{4} cols]]",
                 nrows=rowsize,
                 ncols=colsize,
@@ -752,8 +778,7 @@ class Matrix(BaseType):
             )
 
     def _assign_element(self, resolved_indexes, value):
-        row, _ = resolved_indexes.indices[0]
-        col, _ = resolved_indexes.indices[1]
+        rowidx, colidx = resolved_indexes.indices
         if type(value) is not Scalar:
             try:
                 value = Scalar.from_value(value, name="")
@@ -766,12 +791,14 @@ class Matrix(BaseType):
                     extra_message="Literal scalars also accepted.",
                 )
         # should we cast?
-        call(f"GrB_Matrix_setElement_{value.dtype}", [self, _CScalar(value), row, col])
+        call(
+            f"GrB_Matrix_setElement_{value.dtype}",
+            [self, _CScalar(value), rowidx.index, colidx.index],
+        )
 
     def _prep_for_assign(self, resolved_indexes, value, mask=None, is_submask=False):
         method_name = "__setitem__"
-        rows, rowsize = resolved_indexes.indices[0]
-        cols, colsize = resolved_indexes.indices[1]
+        (rowsize, rows, rowscalar), (colsize, cols, colscalar) = resolved_indexes.indices
         extra_message = "Literal scalars also accepted."
 
         value_type = output_type(value)
@@ -784,7 +811,6 @@ class Matrix(BaseType):
                 )
             if rowsize is None and colsize is not None:
                 # Row-only selection
-                row_index = rows
                 if mask is not None and type(mask.mask) is Matrix:
                     if is_submask:
                         # C[i, J](M) << v
@@ -796,19 +822,16 @@ class Matrix(BaseType):
                         # C(M)[i, J] << v
                         # Upcast v to a Matrix and use Matrix_assign
                         rows = _CArray([rows.scalar.value])
-                        rowsize = _CScalar(1)
-                        new_value = Matrix.new(
-                            value.dtype, nrows=1, ncols=value.size, name=f"{value.name}_as_matrix"
-                        )
-                        new_value[0, :] = value
+                        rowscalar = _CScalar(1)
                         delayed = MatrixExpression(
                             method_name,
                             "GrB_Matrix_assign",
-                            [new_value, rows, rowsize, cols, colsize],
+                            [_VectorAsMatrix(value), rows, rowscalar, cols, colscalar],
                             expr_repr="[[{2} rows], [{4} cols]] = {0.name}",
                             nrows=self._nrows,
                             ncols=self._ncols,
                             dtype=self.dtype,
+                            at=True,
                         )
                 else:
                     if is_submask:
@@ -824,7 +847,7 @@ class Matrix(BaseType):
                     delayed = MatrixExpression(
                         method_name,
                         cfunc_name,
-                        [value, row_index, cols, colsize],
+                        [value, rows, cols, colscalar],
                         expr_repr=expr_repr,
                         nrows=self._nrows,
                         ncols=self._ncols,
@@ -832,7 +855,6 @@ class Matrix(BaseType):
                     )
             elif colsize is None and rowsize is not None:
                 # Column-only selection
-                col_index = cols
                 if mask is not None and type(mask.mask) is Matrix:
                     if is_submask:
                         # C[I, j](M) << v
@@ -844,15 +866,11 @@ class Matrix(BaseType):
                         # C(M)[I, j] << v
                         # Upcast v to a Matrix and use Matrix_assign
                         cols = _CArray([cols.scalar.value])
-                        colsize = _CScalar(1)
-                        new_value = Matrix.new(
-                            value.dtype, nrows=value.size, ncols=1, name=f"{value.name}_as_matrix"
-                        )
-                        new_value[:, 0] = value
+                        colscalar = _CScalar(1)
                         delayed = MatrixExpression(
                             method_name,
                             "GrB_Matrix_assign",
-                            [new_value, rows, rowsize, cols, colsize],
+                            [_VectorAsMatrix(value), rows, rowscalar, cols, colscalar],
                             expr_repr="[[{2} rows], [{4} cols]] = {0.name}",
                             nrows=self._nrows,
                             ncols=self._ncols,
@@ -872,7 +890,7 @@ class Matrix(BaseType):
                     delayed = MatrixExpression(
                         method_name,
                         cfunc_name,
-                        [value, rows, rowsize, col_index],
+                        [value, rows, rowscalar, cols],
                         expr_repr=expr_repr,
                         nrows=self._nrows,
                         ncols=self._ncols,
@@ -932,7 +950,7 @@ class Matrix(BaseType):
             delayed = MatrixExpression(
                 method_name,
                 cfunc_name,
-                [value, rows, rowsize, cols, colsize],
+                [value, rows, rowscalar, cols, colscalar],
                 expr_repr=expr_repr,
                 nrows=self._nrows,
                 ncols=self._ncols,
@@ -973,11 +991,10 @@ class Matrix(BaseType):
                     value_vector << value
 
                     # Row-only selection
-                    row_index = rows
                     delayed = MatrixExpression(
                         method_name,
                         cfunc_name,
-                        [value_vector, row_index, cols, colsize],
+                        [value_vector, rows, cols, colscalar],
                         expr_repr=expr_repr,
                         nrows=self._nrows,
                         ncols=self._ncols,
@@ -998,11 +1015,10 @@ class Matrix(BaseType):
                     value_vector << value
 
                     # Column-only selection
-                    col_index = cols
                     delayed = MatrixExpression(
                         method_name,
                         cfunc_name,
-                        [value_vector, rows, rowsize, col_index],
+                        [value_vector, rows, rowscalar, cols],
                         expr_repr="[[{2} rows], {3}] = {0.name}",
                         nrows=self._nrows,
                         ncols=self._ncols,
@@ -1044,16 +1060,16 @@ class Matrix(BaseType):
                     # C(M)[i, j] << c
                     if rowsize is None:
                         rows = _CArray([rows.scalar.value])
-                        rowsize = _CScalar(1)
+                        rowscalar = _CScalar(1)
                     if colsize is None:
                         cols = _CArray([cols.scalar.value])
-                        colsize = _CScalar(1)
+                        colscalar = _CScalar(1)
                     cfunc_name = f"GrB_Matrix_assign_{value.dtype}"
                     expr_repr = "[[{2} rows], [{4} cols]] = {0}"
                 delayed = MatrixExpression(
                     method_name,
                     cfunc_name,
-                    [_CScalar(value), rows, rowsize, cols, colsize],
+                    [_CScalar(value), rows, rowscalar, cols, colscalar],
                     expr_repr=expr_repr,
                     nrows=self._nrows,
                     ncols=self._ncols,
@@ -1062,9 +1078,8 @@ class Matrix(BaseType):
         return delayed
 
     def _delete_element(self, resolved_indexes):
-        row, _ = resolved_indexes.indices[0]
-        col, _ = resolved_indexes.indices[1]
-        call("GrB_Matrix_removeElement", [self, row, col])
+        rowidx, colidx = resolved_indexes.indices
+        call("GrB_Matrix_removeElement", [self, rowidx.index, colidx.index])
 
     def to_pygraphblas(self):  # pragma: no cover
         """Convert to a new `pygraphblas.Matrix`
@@ -1205,7 +1220,9 @@ class MatrixExpression(BaseExpression):
     name = name.setter(_automethods._set_name)
     nvals = wrapdoc(Matrix.nvals)(property(_automethods.nvals))
     reduce_columns = wrapdoc(Matrix.reduce_columns)(property(_automethods.reduce_columns))
+    reduce_columnwise = wrapdoc(Matrix.reduce_columnwise)(property(_automethods.reduce_columnwise))
     reduce_rows = wrapdoc(Matrix.reduce_rows)(property(_automethods.reduce_rows))
+    reduce_rowwise = wrapdoc(Matrix.reduce_rowwise)(property(_automethods.reduce_rowwise))
     reduce_scalar = wrapdoc(Matrix.reduce_scalar)(property(_automethods.reduce_scalar))
     ss = wrapdoc(Matrix.ss)(property(_automethods.ss))
     to_pygraphblas = wrapdoc(Matrix.to_pygraphblas)(property(_automethods.to_pygraphblas))
@@ -1214,16 +1231,17 @@ class MatrixExpression(BaseExpression):
     # These raise exceptions
     __array__ = wrapdoc(Matrix.__array__)(Matrix.__array__)
     __bool__ = wrapdoc(Matrix.__bool__)(Matrix.__bool__)
-    __iand__ = wrapdoc(Matrix.__iand__)(Matrix.__iand__)
-    __imatmul__ = wrapdoc(Matrix.__imatmul__)(Matrix.__imatmul__)
-    __ior__ = wrapdoc(Matrix.__ior__)(Matrix.__ior__)
     __iadd__ = _automethods.__iadd__
+    __iand__ = _automethods.__iand__
     __ifloordiv__ = _automethods.__ifloordiv__
+    __imatmul__ = _automethods.__imatmul__
     __imod__ = _automethods.__imod__
     __imul__ = _automethods.__imul__
+    __ior__ = _automethods.__ior__
     __ipow__ = _automethods.__ipow__
     __isub__ = _automethods.__isub__
     __itruediv__ = _automethods.__itruediv__
+    __ixor__ = _automethods.__ixor__
 
 
 class TransposedMatrix:
@@ -1246,7 +1264,7 @@ class TransposedMatrix:
 
         return format_matrix_html(self)
 
-    def new(self, *, dtype=None, mask=None, name=None):
+    def new(self, dtype=None, *, mask=None, name=None):
         if dtype is None:
             dtype = self.dtype
         output = Matrix.new(dtype, self._nrows, self._ncols, name=name)
@@ -1271,8 +1289,8 @@ class TransposedMatrix:
         return self._matrix.dtype
 
     @wrapdoc(Matrix.to_values)
-    def to_values(self, *, dtype=None):
-        rows, cols, vals = self._matrix.to_values(dtype=dtype)
+    def to_values(self, dtype=None):
+        rows, cols, vals = self._matrix.to_values(dtype)
         return cols, rows, vals
 
     @property
@@ -1301,6 +1319,8 @@ class TransposedMatrix:
     mxm = Matrix.mxm
     kronecker = Matrix.kronecker
     apply = Matrix.apply
+    reduce_rowwise = Matrix.reduce_rowwise
+    reduce_columnwise = Matrix.reduce_columnwise
     reduce_rows = Matrix.reduce_rows
     reduce_columns = Matrix.reduce_columns
     reduce_scalar = Matrix.reduce_scalar
@@ -1308,22 +1328,23 @@ class TransposedMatrix:
     # Operator sugar
     __or__ = Matrix.__or__
     __ror__ = Matrix.__ror__
-    __ior__ = Matrix.__ior__
     __and__ = Matrix.__and__
     __rand__ = Matrix.__rand__
-    __iand__ = Matrix.__iand__
     __matmul__ = Matrix.__matmul__
     __rmatmul__ = Matrix.__rmatmul__
-    __imatmul__ = Matrix.__imatmul__
 
     # Bad sugar
     __iadd__ = _automethods.__iadd__
+    __iand__ = _automethods.__iand__
     __ifloordiv__ = _automethods.__ifloordiv__
+    __imatmul__ = _automethods.__imatmul__
     __imod__ = _automethods.__imod__
     __imul__ = _automethods.__imul__
+    __ior__ = _automethods.__ior__
     __ipow__ = _automethods.__ipow__
     __isub__ = _automethods.__isub__
     __itruediv__ = _automethods.__itruediv__
+    __ixor__ = _automethods.__ixor__
 
     # Misc.
     isequal = Matrix.isequal
@@ -1343,3 +1364,6 @@ class TransposedMatrix:
 utils._output_types[Matrix] = Matrix
 utils._output_types[MatrixExpression] = Matrix
 utils._output_types[TransposedMatrix] = TransposedMatrix
+
+# Import infix to import _infixmethods, which has side effects
+from . import infix  # noqa isort:skip
