@@ -5,7 +5,7 @@ import numpy as np
 from . import _automethods, backend, ffi, utils
 from .base import BaseExpression, BaseType
 from .binary import isclose
-from .dtypes import _INDEX, lookup_dtype
+from .dtypes import _INDEX, BOOL, lookup_dtype
 from .operator import get_typed_op
 from .utils import output_type, wrapdoc
 
@@ -58,7 +58,7 @@ class Scalar(BaseType):
 
     def __neg__(self):
         dtype = self.dtype
-        if dtype.name[0] == "U" or dtype.name == "BOOL":
+        if dtype.name[0] == "U" or dtype == BOOL:
             raise TypeError(f"The negative operator, `-`, is not supported for {dtype.name} dtype")
         rv = Scalar.new(dtype, name=f"-{self.name}")
         if self.is_empty:
@@ -66,6 +66,18 @@ class Scalar(BaseType):
         rv.value = -self.value
         return rv
 
+    def __invert__(self):
+        if self.dtype != BOOL:
+            raise TypeError(
+                f"The invert operator, `~`, is not supported for {self.dtype.name} dtype.  "
+                "It is only supported for BOOL dtype."
+            )
+        rv = Scalar.new(BOOL, name=f"~{self.name}")
+        if self.is_empty:
+            return rv
+        rv.value = not self.value
+        return rv
+
     __index__ = __int__
 
     def __array__(self, dtype=None):
@@ -132,7 +144,7 @@ class Scalar(BaseType):
             return self.is_empty is other.is_empty
         # We can't yet call a UDF on a scalar as part of the spec, so let's do it ourselves
         isclose_func = isclose(rel_tol, abs_tol)
-        isclose_func = get_typed_op(isclose_func, self.dtype, other.dtype)
+        isclose_func = get_typed_op(isclose_func, self.dtype, other.dtype, kind="binary")
         return isclose_func.numba_func(self.value, other.value)
 
     def clear(self):
@@ -168,7 +180,7 @@ class Scalar(BaseType):
 
     _nvals = nvals
 
-    def dup(self, *, dtype=None, name=None):
+    def dup(self, dtype=None, *, name=None):
         """Create a new Scalar by duplicating this one"""
         if dtype is None:
             new_scalar = Scalar.new(self.dtype, name=name)
@@ -195,9 +207,9 @@ class Scalar(BaseType):
     def from_value(cls, value, dtype=None, *, name=None):
         """Create a new Scalar from a Python value"""
         if type(value) is Scalar:
-            return value.dup(dtype=dtype, name=name)
+            return value.dup(dtype, name=name)
         elif output_type(value) is Scalar:
-            return value.new(dtype=dtype, name=name)
+            return value.new(dtype, name=name)
         elif dtype is None:
             try:
                 dtype = lookup_dtype(type(value))
@@ -214,7 +226,7 @@ class Scalar(BaseType):
 
     @staticmethod
     def _deserialize(value, dtype, name):
-        return Scalar.from_value(value, dtype=dtype, name=name)
+        return Scalar.from_value(value, dtype, name=name)
 
     def to_pygraphblas(self):  # pragma: no cover
         """Convert to a new `pygraphblas.Scalar`
@@ -258,8 +270,8 @@ class ScalarExpression(BaseExpression):
             dtype = self.dtype
         return Scalar.new(dtype, name=name)
 
-    def new(self, *, dtype=None, name=None):
-        return super().new(dtype=dtype, name=name)
+    def new(self, dtype=None, *, name=None):
+        return super().new(dtype, name=name)
 
     dup = new
 
@@ -282,6 +294,7 @@ class ScalarExpression(BaseExpression):
     __float__ = wrapdoc(Scalar.__float__)(property(_automethods.__float__))
     __index__ = wrapdoc(Scalar.__index__)(property(_automethods.__index__))
     __int__ = wrapdoc(Scalar.__int__)(property(_automethods.__int__))
+    __invert__ = wrapdoc(Scalar.__invert__)(property(_automethods.__invert__))
     __neg__ = wrapdoc(Scalar.__neg__)(property(_automethods.__neg__))
     _name_html = wrapdoc(Scalar._name_html)(property(_automethods._name_html))
     _nvals = wrapdoc(Scalar._nvals)(property(_automethods._nvals))
@@ -297,9 +310,6 @@ class ScalarExpression(BaseExpression):
     wait = wrapdoc(Scalar.wait)(property(_automethods.wait))
     # These raise exceptions
     __and__ = wrapdoc(Scalar.__and__)(Scalar.__and__)
-    __iand__ = wrapdoc(Scalar.__iand__)(Scalar.__iand__)
-    __imatmul__ = wrapdoc(Scalar.__imatmul__)(Scalar.__imatmul__)
-    __ior__ = wrapdoc(Scalar.__ior__)(Scalar.__ior__)
     __matmul__ = wrapdoc(Scalar.__matmul__)(Scalar.__matmul__)
     __or__ = wrapdoc(Scalar.__or__)(Scalar.__or__)
     __rand__ = wrapdoc(Scalar.__rand__)(Scalar.__rand__)
@@ -317,7 +327,7 @@ class _CScalar:
 
     def __init__(self, scalar, dtype=_INDEX):
         if type(scalar) is not Scalar:
-            scalar = Scalar.from_value(scalar, name="", dtype=dtype)
+            scalar = Scalar.from_value(scalar, dtype, name="")
         self.scalar = scalar
         self.dtype = scalar.dtype
 
