@@ -9,7 +9,7 @@ def test_import_special_attrs():
     assert len(not_hidden & grblas._SPECIAL_ATTRS) == len(grblas._SPECIAL_ATTRS)
     # Is everything special that needs to be?
     not_special = {x for x in dir(grblas) if not x.startswith("_")} - grblas._SPECIAL_ATTRS
-    assert not_special == {"backend", "backends", "init", "mask", "replace"}
+    assert not_special == {"backend", "backends", "config", "init", "mask", "replace"}
     # Make sure these "not special" objects don't have objects that look special within them
     for attr in not_special:
         assert not set(dir(getattr(grblas, attr))) & grblas._SPECIAL_ATTRS
