@@ -33,20 +33,24 @@ def s1():
 
 @autocompute
 def test_ewise(v1, v2, A1, A2):
+    v1 = v1.dup(dtype=bool)
+    v2 = v2.dup(dtype=bool)
+    A1 = A1.dup(dtype=bool)
+    A2 = A2.dup(dtype=bool)
     for left, right in [
         (v1, v2),
         (A1, A2.T),
         (A1.T, A1.T),
         (A1, A1),
     ]:
-        expected = left.ewise_mult(right, monoid.times).new()
+        expected = left.ewise_mult(right, monoid.land).new()
         expr = left & right
         assert expr.nvals == expected.nvals
         val = expr.new(name="val")
         assert val.name == "val"
         assert expected.isequal((left & right).new(dtype=float))
-        assert expected.isequal(monoid.times(left & right).new())
-        assert expected.isequal(monoid.times[float](left & right).new())
+        assert expected.isequal(monoid.land(left & right).new())
+        assert expected.isequal(monoid.land[float](left & right).new())
         assert expected.isequal((left & right).new())  # use `left.ewise_mult` default op
         if isinstance(left, Vector):
             assert (left & right).size == left.size
@@ -57,9 +61,9 @@ def test_ewise(v1, v2, A1, A2):
             assert (left & right).ncols == left.ncols
             assert (left | right).ncols == left.ncols
 
-        expected = left.ewise_add(right, op.plus).new()
-        assert expected.isequal(op.plus(left | right).new())
-        assert expected.isequal(op.plus[float](left | right).new())
+        expected = left.ewise_add(right, op.lor).new()
+        assert expected.isequal(op.lor(left | right).new())
+        assert expected.isequal(op.lor[float](left | right).new())
         assert expected.isequal((left | right).new())  # use `left.ewise_add` default op
 
         expected = left.ewise_mult(right, op.minus).new()
@@ -67,6 +71,8 @@ def test_ewise(v1, v2, A1, A2):
 
         expected = left.ewise_add(right, op.minus, require_monoid=False).new()
         assert expected.isequal(op.minus(left | right, require_monoid=False).new())
+        expr = left | right
+        assert expr.nvals == expr.nvals  # tests caching of ._value
 
 
 def test_matmul(v1, v2, A1, A2):
@@ -144,21 +150,21 @@ def test_bad_ewise(s1, v1, A1, A2):
     with raises(TypeError):
         1 & s1
 
-    with raises(TypeError, match="Using __ior__"):
+    with raises(TypeError, match="not supported for FP64"):
         v1 |= v1
-    with raises(TypeError, match="Using __ior__"):
+    with raises(TypeError, match="not supported for FP64"):
         A1 |= A1
-    with raises(TypeError, match="Using __iand__"):
+    with raises(TypeError, match="not supported for FP64"):
         v1 &= v1
-    with raises(TypeError, match="Using __iand__"):
+    with raises(TypeError, match="not supported for FP64"):
         A1 &= A1
 
     with raises(TypeError, match="require_monoid"):
         op.minus(v1 | v1)
     with raises(TypeError):
         op.minus(v1 & v1, require_monoid=False)
-    # with raises(TypeError, match="Bad types when calling binary.plus"):
-    op.plus(v1 & v1, 1)  # Now okay
+    with raises(TypeError, match="Bad dtype"):
+        op.plus(v1 & v1, 1)
 
 
 def test_bad_matmul(s1, v1, A1, A2):
@@ -189,7 +195,7 @@ def test_bad_matmul(s1, v1, A1, A2):
         A1 @ A1
     with raises(DimensionMismatch):
         A1.T @ A1.T
-    with raises(TypeError, match="__imatmul__"):
+    with raises(DimensionMismatch):
         A1 @= A1
     with raises(TypeError):
         s1 @ 1
@@ -199,7 +205,7 @@ def test_bad_matmul(s1, v1, A1, A2):
     w = v1[:1].new()
     with raises(DimensionMismatch):
         w @ v1
-    with raises(TypeError, match="__imatmul__"):
+    with raises(DimensionMismatch):
         v1 @= v1
 
     with raises(TypeError, match="Bad type when calling semiring.plus_times"):
@@ -229,8 +235,8 @@ def test_apply_unary_bad(s1, v1):
         op.exp(s1)
     with raises(TypeError, match="Bad type when calling unary.exp"):
         op.exp(1)
-    # with raises(TypeError, match="Bad type when calling unary.exp"):
-    op.exp(v1 | v1)  # Now okay
+    with raises(TypeError, match="Bad dtype"):
+        op.exp(v1 | v1)
 
 
 def test_apply_binary(v1, A1):
@@ -261,3 +267,53 @@ def test_apply_binary_bad(s1, v1):
         op.plus(v1, v1)
     with raises(TypeError, match="may only be used when performing an ewise_add"):
         op.plus(v1, 1, require_monoid=False)
+
+
+def test_infix_nonscalars(v1, v2):
+    with raises(TypeError, match="refuse to guess"):
+        v1 + v2
+
+
+@autocompute
+def test_inplace_infix(s1, v1, v2, A1, A2):
+    A = Matrix.new(float, nrows=3, ncols=3)
+    A[:, :] = 1
+    x = v1.dup()
+    x @= A
+    assert isinstance(x, Vector)
+    assert x.isequal(v1 @ A)
+    with raises(TypeError, match="not supported for FP64"):
+        v1 |= v2
+    with raises(TypeError, match="not supported for FP64"):
+        A1 &= A2.T
+
+    v1 = v1.dup(bool)
+    v2 = v2.dup(bool)
+    A1 = A1.dup(bool)
+    A2 = A2.dup(bool)
+    x = v1.dup()
+    x |= v2
+    assert isinstance(x, Vector)
+    assert x.isequal(v1 | v2)
+    x = v1.dup()
+    x &= v2
+    assert isinstance(x, Vector)
+    assert x.isequal(v1 & v2)
+    x = A1.dup()
+    x |= A2.T
+    assert isinstance(x, Matrix)
+    assert x.isequal(A1 | A2.T)
+    x = A1.dup()
+    x &= A2.T
+    assert isinstance(x, Matrix)
+    assert x.isequal(A1 & A2.T)
+
+    expr = v1 | v2
+    with raises(TypeError, match="not supported"):
+        expr |= v1
+    with raises(TypeError, match="not supported"):
+        expr &= v1
+    with raises(TypeError, match="not supported"):
+        expr @= A
+    with raises(TypeError, match="not supported"):
+        s1 @= v1
