@@ -146,7 +146,7 @@ class Vector(BaseType):
             return False
 
         matches = self.ewise_mult(other, binary.isclose(rel_tol, abs_tol)).new(
-            dtype=bool, name="M_isclose"
+            bool, name="M_isclose"
         )
         # ewise_mult performs intersection, so nvals will indicate mismatched empty values
         if matches._nvals != self._nvals:
@@ -188,7 +188,7 @@ class Vector(BaseType):
         call("GrB_Vector_resize", [self, size])
         self._size = size.scalar.value
 
-    def to_values(self, *, dtype=None):
+    def to_values(self, dtype=None):
         """
         GrB_Vector_extractTuples
         Extract the indices and values as a 2-tuple of numpy arrays
@@ -231,21 +231,21 @@ class Vector(BaseType):
         dup_op_given = dup_op is not None
         if not dup_op_given:
             dup_op = binary.plus
-        dup_op = get_typed_op(dup_op, self.dtype)
+        dup_op = get_typed_op(dup_op, self.dtype, kind="binary")
         if dup_op.opclass == "Monoid":
             dup_op = dup_op.binaryop
         else:
             self._expect_op(dup_op, "BinaryOp", within="build", argname="dup_op")
 
         indices = _CArray(indices)
-        values = _CArray(values, dtype=self.dtype)
+        values = _CArray(values, self.dtype)
         call(f"GrB_Vector_build_{self.dtype.name}", [self, indices, values, _CScalar(n), dup_op])
 
         # Check for duplicates when dup_op was not provided
         if not dup_op_given and self._nvals < n:
             raise ValueError("Duplicate indices found, must provide `dup_op` BinaryOp")
 
-    def dup(self, *, dtype=None, mask=None, name=None):
+    def dup(self, dtype=None, *, mask=None, name=None):
         """
         GrB_Vector_dup
         Create a new Vector by duplicating this one
@@ -254,7 +254,7 @@ class Vector(BaseType):
             if dtype is None:
                 dtype = self.dtype
             rv = Vector.new(dtype, size=self._size, name=name)
-            rv(mask=mask)[:] << self
+            rv(mask=mask)[...] = self
         else:
             new_vec = ffi_new("GrB_Vector*")
             rv = Vector(new_vec, self.dtype, name=name)
@@ -281,14 +281,13 @@ class Vector(BaseType):
         new_vector = ffi_new("GrB_Vector*")
         dtype = lookup_dtype(dtype)
         rv = cls(new_vector, dtype, name=name)
-        if type(size) is not _CScalar:
-            size = _CScalar(size)
+        size = _CScalar(size)
         call("GrB_Vector_new", [_Pointer(rv), dtype, size])
         rv._size = size.scalar.value
         return rv
 
     @classmethod
-    def from_values(cls, indices, values, *, size=None, dup_op=None, dtype=None, name=None):
+    def from_values(cls, indices, values, dtype=None, *, size=None, dup_op=None, name=None):
         """Create a new Vector from the given lists of indices and values.  If
         size is not provided, it is computed from the max index found.
 
@@ -350,7 +349,7 @@ class Vector(BaseType):
         """
         method_name = "ewise_add"
         other = self._expect_type(other, Vector, within=method_name, argname="other")
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         # Per the spec, op may be a semiring, but this is weird, so don't.
         if require_monoid:
             if op.opclass != "BinaryOp" or op.monoid is None:
@@ -359,7 +358,7 @@ class Vector(BaseType):
                     "Monoid",
                     within=method_name,
                     argname="op",
-                    extra_message="A BinaryOp may be given if require_monoid keyword is False",
+                    extra_message="A BinaryOp may be given if require_monoid keyword is False.",
                 )
         else:
             self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
@@ -382,7 +381,7 @@ class Vector(BaseType):
         """
         method_name = "ewise_mult"
         other = self._expect_type(other, Vector, within=method_name, argname="other")
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         # Per the spec, op may be a semiring, but this is weird, so don't.
         self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
         expr = VectorExpression(
@@ -407,7 +406,7 @@ class Vector(BaseType):
         other = self._expect_type(
             other, (Matrix, TransposedMatrix), within=method_name, argname="other"
         )
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="semiring")
         self._expect_op(op, "Semiring", within=method_name, argname="op")
         expr = VectorExpression(
             method_name,
@@ -421,7 +420,7 @@ class Vector(BaseType):
             expr.new(name="")  # incompatible shape; raise now
         return expr
 
-    def apply(self, op, *, left=None, right=None):
+    def apply(self, op, right=None, *, left=None):
         """
         GrB_Vector_apply
         Apply UnaryOp to each element of the calling Vector
@@ -433,7 +432,7 @@ class Vector(BaseType):
             "apply only accepts UnaryOp with no scalars or BinaryOp with `left` or `right` scalar."
         )
         if left is None and right is None:
-            op = get_typed_op(op, self.dtype)
+            op = get_typed_op(op, self.dtype, kind="unary")
             self._expect_op(
                 op,
                 "UnaryOp",
@@ -456,7 +455,7 @@ class Vector(BaseType):
                         keyword_name="left",
                         extra_message="Literal scalars also accepted.",
                     )
-            op = get_typed_op(op, self.dtype, left.dtype)
+            op = get_typed_op(op, self.dtype, left.dtype, kind="binary")
             if op.opclass == "Monoid":
                 op = op.binaryop
             else:
@@ -482,7 +481,7 @@ class Vector(BaseType):
                         keyword_name="right",
                         extra_message="Literal scalars also accepted.",
                     )
-            op = get_typed_op(op, self.dtype, right.dtype)
+            op = get_typed_op(op, self.dtype, right.dtype, kind="binary")
             if op.opclass == "Monoid":
                 op = op.binaryop
             else:
@@ -514,7 +513,7 @@ class Vector(BaseType):
         Default op is monoid.lor for boolean and monoid.plus otherwise
         """
         method_name = "reduce"
-        op = get_typed_op(op, self.dtype)
+        op = get_typed_op(op, self.dtype, kind="binary")
         if op.opclass == "BinaryOp" and op.monoid is not None:
             op = op.monoid
         else:
@@ -537,7 +536,7 @@ class Vector(BaseType):
         """
         method_name = "inner"
         other = self._expect_type(other, Vector, within=method_name, argname="other")
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="semiring")
         self._expect_op(op, "Semiring", within=method_name, argname="op")
         expr = ScalarExpression(
             method_name,
@@ -561,7 +560,7 @@ class Vector(BaseType):
 
         method_name = "outer"
         other = self._expect_type(other, Vector, within=method_name, argname="other")
-        op = get_typed_op(op, self.dtype, other.dtype)
+        op = get_typed_op(op, self.dtype, other.dtype, kind="binary")
         self._expect_op(op, ("BinaryOp", "Monoid"), within=method_name, argname="op")
         if op.opclass == "Monoid":
             op = op.binaryop
@@ -585,28 +584,28 @@ class Vector(BaseType):
             dtype = self.dtype
         else:
             dtype = lookup_dtype(dtype)
-        index, _ = resolved_indexes.indices[0]
+        idx = resolved_indexes.indices[0]
         result = Scalar.new(dtype, name=name)
         if (
-            call(f"GrB_Vector_extractElement_{dtype}", [_Pointer(result), self, index])
+            call(f"GrB_Vector_extractElement_{dtype}", [_Pointer(result), self, idx.index])
             is not NoValue
         ):
             result._is_empty = False
         return result
 
     def _prep_for_extract(self, resolved_indexes, mask=None, is_submask=False):
-        index, isize = resolved_indexes.indices[0]
+        size, index, cscalar = resolved_indexes.indices[0]
         return VectorExpression(
             "__getitem__",
             "GrB_Vector_extract",
-            [self, index, isize],
+            [self, index, cscalar],
             expr_repr="{0.name}[[{2} elements]]",
-            size=isize,
+            size=size,
             dtype=self.dtype,
         )
 
     def _assign_element(self, resolved_indexes, value):
-        index, _ = resolved_indexes.indices[0]
+        idx = resolved_indexes.indices[0]
         if type(value) is not Scalar:
             try:
                 value = Scalar.from_value(value, name="")
@@ -619,11 +618,11 @@ class Vector(BaseType):
                     extra_message="Literal scalars also accepted.",
                 )
         # should we cast?
-        call(f"GrB_Vector_setElement_{value.dtype}", [self, _CScalar(value), index])
+        call(f"GrB_Vector_setElement_{value.dtype}", [self, _CScalar(value), idx.index])
 
     def _prep_for_assign(self, resolved_indexes, value, mask=None, is_submask=False):
         method_name = "__setitem__"
-        index, isize = resolved_indexes.indices[0]
+        size, index, cscalar = resolved_indexes.indices[0]
 
         if output_type(value) is Vector:
             if type(value) is not Vector:
@@ -633,7 +632,7 @@ class Vector(BaseType):
                     within=method_name,
                 )
             if is_submask:
-                if isize is None:
+                if size is None:
                     # v[i](m) << w
                     raise TypeError("Single element assign does not accept a submask")
                 # v[I](m) << w
@@ -658,7 +657,7 @@ class Vector(BaseType):
                     )
             value = _CScalar(value)
             if is_submask:
-                if isize is None:
+                if size is None:
                     # v[i](m) << c
                     raise TypeError("Single element assign does not accept a submask")
                 # v[I](m) << c
@@ -667,23 +666,23 @@ class Vector(BaseType):
             else:
                 # v(m)[I] << c
                 # v[I] << c
-                if isize is None:
+                if size is None:
                     index = _CArray([index.scalar.value])
-                    isize = _CScalar(1)
+                    cscalar = _CScalar(1)
                 cfunc_name = f"GrB_Vector_assign_{value.dtype}"
                 expr_repr = "[[{2} elements]] = {0}"
         return VectorExpression(
             method_name,
             cfunc_name,
-            [value, index, isize],
+            [value, index, cscalar],
             expr_repr=expr_repr,
             size=self._size,
             dtype=self.dtype,
         )
 
     def _delete_element(self, resolved_indexes):
-        index, _ = resolved_indexes.indices[0]
-        call("GrB_Vector_removeElement", [self, index])
+        idx = resolved_indexes.indices[0]
+        call("GrB_Vector_removeElement", [self, idx.index])
 
     def to_pygraphblas(self):  # pragma: no cover
         """Convert to a new `pygraphblas.Vector`
@@ -820,16 +819,17 @@ class VectorExpression(BaseExpression):
     # These raise exceptions
     __array__ = wrapdoc(Vector.__array__)(Vector.__array__)
     __bool__ = wrapdoc(Vector.__bool__)(Vector.__bool__)
-    __iand__ = wrapdoc(Vector.__iand__)(Vector.__iand__)
-    __imatmul__ = wrapdoc(Vector.__imatmul__)(Vector.__imatmul__)
-    __ior__ = wrapdoc(Vector.__ior__)(Vector.__ior__)
     __iadd__ = _automethods.__iadd__
+    __iand__ = _automethods.__iand__
     __ifloordiv__ = _automethods.__ifloordiv__
+    __imatmul__ = _automethods.__imatmul__
     __imod__ = _automethods.__imod__
     __imul__ = _automethods.__imul__
+    __ior__ = _automethods.__ior__
     __ipow__ = _automethods.__ipow__
     __isub__ = _automethods.__isub__
     __itruediv__ = _automethods.__itruediv__
+    __ixor__ = _automethods.__ixor__
 
 
 class _VectorAsMatrix:
@@ -859,3 +859,6 @@ class _VectorAsMatrix:
 
 utils._output_types[Vector] = Vector
 utils._output_types[VectorExpression] = Vector
+
+# Import matrix to import infix to import _infixmethods, which has side effects
+from . import matrix  # noqa isort:skip
